/*
* Copyright (C) 2003-2007 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "resource.h"
#include "CalendarSettings.h"
#include "MainSyncFrm.h"
#include "ClientUtil.h"

#include "winmaincpp.h"
#include "utils.h"
#include "comutil.h"
#include "OutlookPlugin.h"

#include <string>

using namespace std;

static wstring outlookFolder;
static CCalendarSettings* wndCalendar;
static HANDLE handleThread;

IMPLEMENT_DYNCREATE(CCalendarSettings, CDialog)

CCalendarSettings::CCalendarSettings()
	: CDialog(CCalendarSettings::IDD)
{
}

CCalendarSettings::~CCalendarSettings()
{
    // clean stuff used in the select Outlook folder thread
    wndCalendar = NULL;
    CloseHandle(handleThread);
}

void CCalendarSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_CALENDAR_COMBO_SYNCTYPE, lstSyncType);
    DDX_Control(pDX, IDC_CALENDAR_EDIT_FOLDER, editFolder);
    DDX_Control(pDX, IDC_CALENDAR_CHECK_INCLUDE, checkIncludeSubfolders);
    DDX_Control(pDX, IDC_CALENDAR_BUT_SELECT, butSelectFolder);
    DDX_Control(pDX, IDC_CALENDAR_EDIT_REMOTE, editRemote);
    //DDX_Control(pDX, IDC_CALENDAR_APPLY, butApply);
    DDX_Control(pDX, IDC_CALENDAR_RADIO_SIF, radioSif);
    DDX_Control(pDX, IDC_CALENDAR_RADIO_VCARD, radioVcard);
    DDX_Control(pDX, IDC_CALENDAR_GROUP_DIRECTION, groupDirection);
    DDX_Control(pDX, IDC_CALENDAR_GROUP_FOLDER, groupFolder);
    DDX_Control(pDX, IDC_CALENDAR_GROUP_ADVANCED, groupAdvanced);
}

BEGIN_MESSAGE_MAP(CCalendarSettings, CDialog)
    ON_BN_CLICKED(IDC_CALENDAR_BUTOK, &CCalendarSettings::OnBnClickedCalendarButok)
    ON_BN_CLICKED(IDC_CALENDAR_BUTCANCEL, &CCalendarSettings::OnBnClickedCalendarButcancel)
    ON_BN_CLICKED(IDC_CALENDAR_BUT_SELECT, &CCalendarSettings::OnBnClickedCalendarButSelect)
    ON_WM_SYSCOMMAND()
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()

#ifdef _DEBUG
void CCalendarSettings::AssertValid() const
{
	CDialog::AssertValid();
}

#ifndef _WIN32_WCE
void CCalendarSettings::Dump(CDumpContext& dc) const
{
	CDialog::Dump(dc);
}
#endif
#endif //_DEBUG


// CCalendarSettings message handlers

BOOL CCalendarSettings::OnInitDialog(){
    CString s1;
    s1.LoadString(IDS_CALENDAR_DETAILS); SetWindowText(s1);
    CDialog::OnInitDialog();
    
    
    WindowsSyncSourceConfig* ssconf = ((OutlookConfig*)getConfig())->getSyncSourceConfig(APPOINTMENT_);
    
    editFolder.SetLimitText(EDIT_TEXT_MAXLENGTH);
    editRemote.SetLimitText(EDIT_TEXT_MAXLENGTH);
    
    groupDirection.id = 2; groupDirection.text.LoadString(IDS_SYNC_DIRECTION);
    groupFolder.id = 2; groupFolder.text.LoadString(IDS_CALENDAR_FOLDER);
    groupAdvanced.id = 2; groupAdvanced.text.LoadString(IDS_ADVANCED);
    // load string resources
    s1.LoadString(IDS_SYNCTYPE1); lstSyncType.AddString(s1);
    s1.LoadString(IDS_SYNCTYPE2); lstSyncType.AddString(s1);
    s1.LoadString(IDS_SYNCTYPE3); lstSyncType.AddString(s1);

    s1.LoadString(IDS_SYNC_DIRECTION); SetDlgItemText(IDC_CALENDAR_GROUP_DIRECTION, s1);
    s1.LoadString(IDS_CURRENT); SetDlgItemText(IDC_CALENDAR_STATIC_FOLDER, s1);
    s1.LoadString(IDS_INCLUDE_SUBFOLDERS); SetDlgItemText(IDC_CALENDAR_CHECK_INCLUDE, s1);
    s1.LoadString(IDS_SELECT_FOLDER); SetDlgItemText(IDC_CALENDAR_BUT_SELECT, s1);
    s1.LoadString(IDS_REMOTE_NAME); SetDlgItemText(IDC_CALENDAR_STATIC_REMOTE, s1);
    s1.LoadString(IDS_ADVANCED); SetDlgItemText(IDC_CALENDAR_GROUP_ADVANCED, s1);
    s1.LoadString(IDS_CALENDAR_FOLDER); SetDlgItemText(IDC_CALENDAR_GROUP_FOLDER, s1);

    s1.LoadString(IDS_OK); SetDlgItemText(IDC_CALENDAR_BUTOK, s1);
    s1.LoadString(IDS_CANCEL); SetDlgItemText(IDC_CALENDAR_BUTCANCEL, s1);

    lstSyncType.SetCurSel(getSyncTypeIndex(ssconf->getSync()));
    
    // get folder path
    s1 = ssconf->getFolderPath(); 
    if(s1 == ""){
        s1 = getDefaultFolderPath(APPOINTMENT).data();
    }
    SetDlgItemText(IDC_CALENDAR_EDIT_FOLDER, s1);

    if(ssconf->getUseSubfolders())
        checkIncludeSubfolders.SetCheck(BST_CHECKED);
    s1 = ssconf->getURI(); SetDlgItemText(IDC_CALENDAR_EDIT_REMOTE, s1);

    s1.LoadString(IDS_DATA_FORMAT); SetDlgItemText(IDC_CALENDAR_STATIC_DATAFORMAT, s1);
    s1.LoadString(IDS_USE_SIF); SetDlgItemText(IDC_CALENDAR_RADIO_SIF, s1);
    s1.LoadString(IDS_USE_VCAL); SetDlgItemText(IDC_CALENDAR_RADIO_VCARD, s1);

    if( strstr(ssconf->getType(),"sif") ){
        radioSif.SetCheck(BST_CHECKED);
    }
    else
        radioVcard.SetCheck(BST_CHECKED);

    // disable windows xp theme, otherwise any color setting for groupbox
    // will be overriden by the theme settings
    if(((COutlookPluginApp*)AfxGetApp())->hLib){
        PFNSETWINDOWTHEME pfnSetWindowTheme =
            (PFNSETWINDOWTHEME)GetProcAddress(((COutlookPluginApp*)AfxGetApp())->hLib, "SetWindowTheme");
        pfnSetWindowTheme (groupDirection.m_hWnd,L" ",L" ");
        pfnSetWindowTheme (groupFolder.m_hWnd,L" ",L" ");
        pfnSetWindowTheme (groupAdvanced.m_hWnd,L" ",L" ");
    };

    // Accessing Outlook, could be no more in foreground
    SetForegroundWindow();

    wndCalendar = this;
    return FALSE;
}

void CCalendarSettings::OnBnClickedCalendarButok()
{
    // OK Button
    if(saveSettings(false)){
        CDialog::OnOK();
    }
}

void CCalendarSettings::OnBnClickedCalendarButcancel()
{
    // CANCEL button
    getConfig()->read();
    CDialog::OnCancel();
}


bool CCalendarSettings::saveSettings(bool saveToDisk)
{
    CString remoteName, outlookFolder, syncType;
    CString s1;
    _bstr_t bst;
    bool useSif;
    WindowsSyncSourceConfig* ssconf = ((OutlookConfig*)getConfig())->getSyncSourceConfig(APPOINTMENT_);

    GetDlgItemText(IDC_CALENDAR_EDIT_REMOTE, remoteName);
    GetDlgItemText(IDC_CALENDAR_EDIT_FOLDER, outlookFolder);

    // change values
        if(remoteName == ""){
            s1.LoadString(IDS_ERROR_SET_REMOTE_NAME);
            MessageBox(s1);
            return false;
        };
        // sync source enabled
        ssconf->setSync(getSyncTypeName(lstSyncType.GetCurSel()));
        bst.Assign(outlookFolder.AllocSysString());
        ssconf->setFolderPath(bst); bst.Detach(); 

        if(checkIncludeSubfolders.GetCheck() == BST_CHECKED)
            ssconf->setUseSubfolders(true);
        else
            ssconf->setUseSubfolders(false);

        bst.Assign(remoteName.AllocSysString());
        ssconf->setURI(bst); bst.Detach();

        if(radioSif.GetCheck() == BST_CHECKED)
            useSif = true;
        else
            useSif = false;

        // TODO: data formats
        if(useSif){
            ssconf->setType("text/x-s4j-sife");
            ssconf->setVersion("");                     // TODO: Change when SIF version is ready - get from a define.
            ssconf->setEncoding("b64");
        }
        else{
            ssconf->setType("text/x-vcalendar"); 
            ssconf->setVersion(ITEM_VERSION_2_1);                  // TODO: get from a define.
            // When encryption is used, encoding is always 'base64'.
            if ( !strcmp(ssconf->getEncryption(), "") ) {
            ssconf->setEncoding("bin");
            }
            else {
                ssconf->setEncoding("b64");
            }
        }

    if(saveToDisk)
        ((OutlookConfig*)getConfig())->save();

    return true;
}

int pickFolderCalendar(){
    CString s1;
    outlookFolder = pickOutlookFolder(APPOINTMENT);
    if ( (outlookFolder != L"") && (wndCalendar != NULL) ){
      s1 = outlookFolder.data();
      wndCalendar->SetDlgItemText(IDC_CALENDAR_EDIT_FOLDER, s1);
    }
    wndCalendar->SetForegroundWindow();
    return 0;
}
void CCalendarSettings::OnBnClickedCalendarButSelect()
{
    outlookFolder = L"";
    handleThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)pickFolderCalendar, NULL, 0, NULL);
}
void CCalendarSettings::OnSysCommand(UINT nID, LPARAM lParam)
{
    groupAdvanced.Invalidate(false);
    groupFolder.Invalidate(false);

    CDialog::OnSysCommand(nID, lParam);
}

HBRUSH CCalendarSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    if(pWnd->GetRuntimeClass() == RUNTIME_CLASS(CCustomGroupbox)){
            // do not check the id
            pDC->SetBkMode(TRANSPARENT);
            pDC->SetTextColor(COLOR_GROUPBOX_BLACK);
        }

    return hbr;
}
