/*
* Copyright (C) 2003-2007 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "OutlookPlugin.h"
#include "OutlookPlugindoc.h"
#include "LeftView.h"
#include "SyncForm.h"
#include "MainSyncFrm.h"
#include "AccountSettings.h"
#include "FullSync.h"
#include "LogSettings.h"
#include "AnimatedIcon.h"

#include "ClientUtil.h"
#include "utils.h"

#include "HwndFunctions.h"
#include "comutil.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "winmaincpp.h"

#include "Tlhelp32.h"

/////////////////////////////////////////////////////////////////////////////

IMPLEMENT_DYNCREATE(CMainSyncFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainSyncFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CConfigFrame)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code !
    ON_WM_CREATE()
    ON_WM_SIZE()
	//}}AFX_MSG_MAP
    ON_MESSAGE(ID_MYMSG_SYNC_BEGIN, &CMainSyncFrame::OnMsgSyncBegin)
    ON_MESSAGE(ID_MYMSG_SYNC_END, &CMainSyncFrame::OnMsgSyncEnd)
    ON_MESSAGE(ID_MYMSG_SYNCSOURCE_BEGIN, &CMainSyncFrame::OnMsgSyncSourceBegin)
    ON_MESSAGE(ID_MYMSG_SYNCSOURCE_END, &CMainSyncFrame::OnMsgSyncSourceEnd)
    ON_MESSAGE(ID_MYMSG_SYNC_ITEM_SYNCED, &CMainSyncFrame::OnMsgItemSynced)
    ON_MESSAGE(ID_MYMSG_SYNC_TOTALITEMS, &CMainSyncFrame::OnMsgTotalItems) 
    ON_MESSAGE(ID_MYMSG_SYNC_TOTALSOURCES, &CMainSyncFrame::OnMsgTotalSources) 
    ON_MESSAGE(ID_MYMSG_STARTSYNC_ENDED, &CMainSyncFrame::OnMsgStartsyncEnded) 
    ON_MESSAGE(ID_MYMSG_REFRESH_STATUSBAR, &CMainSyncFrame::OnMsgRefreshStatusBar) 
    ON_MESSAGE(ID_MYMSG_SOURCE_STATE, &CMainSyncFrame::OnMsgSyncSourceState) 
    ON_MESSAGE(ID_MYMSG_MODAL   , &CMainSyncFrame::OnMsgAppModal) 
    ON_COMMAND(ID_FILE_CONFIGURATION, &CMainSyncFrame::OnFileConfiguration)
    ON_COMMAND(ID_TOOLS_FULLSYNC, &CMainSyncFrame::OnToolsFullSync)
    ON_COMMAND(ID_FILE_SYNCHRONIZE, &CMainSyncFrame::OnFileSynchronize)
    ON_COMMAND(ID_TOOLS_SETLOGLEVEL, &CMainSyncFrame::OnToolsSetloglevel)
    ON_WM_NCACTIVATE()
    ON_WM_CLOSE()
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};


// Flag used to lock/unlock the statusbar (and other objects).
// During canceling sync, we don't want bar to be updated / object enabled.
bool cancelingSync = false;


/**
 * Function used to refresh the statusbar.
 * Statusbar is not updated if locked by the flag 'cancelingSync'.
 */
void refreshStatusBar(CString& msg) {

    // Avoid updating the statusbar when canceling sync.
    if (!cancelingSync && msg.GetLength()) {
        ((CMainSyncFrame*)AfxGetMainWnd())->wndStatusBar.SetPaneText(0, msg);
    }
}


/////////////////////////////////////////////////////////////////////////////

CMainSyncFrame::CMainSyncFrame() {
    hSyncThread = NULL;
    dwThreadId  = NULL;
    configOpened = false;
    cancelingSync = false;
    
    syncModeContacts = -1; syncModeCalendar = -1; 
    syncModeTasks = -1; syncModeNotes = -1; 
}

CMainSyncFrame::~CMainSyncFrame() {
    if (dwThreadId) {
        CloseHandle(hSyncThread);
    } 
    //closeClient();
}

int CMainSyncFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	if (!wndStatusBar.Create(this) ||
		!wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

    // TODO: hide splitter here
    EnableDocking(CBRS_ALIGN_ANY);
    wndSplitter.SetActivePane(0,1);
    wndSplitter.SetColumnInfo(0,0,0); 
    RecalcLayout();
    SetWindowText(PLUGIN_UI_TITLE); 

    bSyncStarted = false;

	return 0;
}

BOOL CMainSyncFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;

    // TODO: set here main window size and style
    cs.style =  WS_SYSMENU  | WS_VISIBLE | WS_MINIMIZEBOX;
    // cs.dwExStyle = 0 ; 
    int dpiX, dpiY;
    dpiX = ::GetDeviceCaps(::GetDC(0),LOGPIXELSX);
    dpiY = ::GetDeviceCaps(::GetDC(0),LOGPIXELSY);
    double dx = FRAME_MAIN_X * ((double)dpiX/96);      // default DPI = 96
    double dy = FRAME_MAIN_Y * ((double)dpiY/96);      // default DPI = 96
    cs.cx = (int)dx;
    cs.cy = (int)dy;


    // Center window
    cs.x = (GetSystemMetrics(SM_CXSCREEN) - cs.cx)/2;
    cs.y = (GetSystemMetrics(SM_CYSCREEN) - cs.cy)/2;

    // Set the class name, previously registered to be now used.
    // Class name is important to correctly use FindWindow() function.
    cs.lpszClass = PLUGIN_UI_CLASSNAME;

    return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// diagnostics

#ifdef _DEBUG
void CMainSyncFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainSyncFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////

BOOL CMainSyncFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext) 
{ 
    if (!wndSplitter.CreateStatic(this,1,2,WS_CHILD | WS_VISIBLE | WS_MINIMIZEBOX))
	{
		TRACE(_T("failed to create the splitter"));
		return FALSE;
	}
	
    if (!wndSplitter.CreateView(0,0,RUNTIME_CLASS(CSyncForm),CSize(100,100),pContext))
	{
		TRACE(_T("Failed to create view in first pane"));
		return FALSE;
	}

    if (!wndSplitter.CreateView(0,1,RUNTIME_CLASS(CSyncForm),CSize(100,100),pContext))
	{
		TRACE(_T("failed to create view in second pane"));
		return FALSE;
	} 

	return TRUE;
}


void CMainSyncFrame::OnSize(UINT nType, int cx, int cy) 
{
    // TODO: to be removed
	CFrameWnd::OnSize(nType, cx, cy);
}

void CMainSyncFrame::OnFileConfiguration()
{
    // show config settings
    showSettingsWindow();
}


void CMainSyncFrame::OnToolsFullSync()
{
    //-- show full sync dialog
    CFullSync wndFullSync;
    INT_PTR result = wndFullSync.DoModal();  
}



/**
 * Thread to start the sync process.
 */
DWORD WINAPI syncThread(LPVOID lpParam) {
    int ret = startSync();
    return ret;
}


/**
 * Thread used to control the syncThread execution / exit mode.
 * When sync thread has exited, sends a message to UI to manage
 * error and refresh main form.
 * Also adds a timeout of SYNC_TIMEOUT minutes to the sync process, 
 * after that the sync is aborted.
 *
 * @param lpParam : the syncThread HANDLE
 */
DWORD WINAPI syncThreadController(LPVOID lpParam) {

    HANDLE hSyncThread = lpParam;
    DWORD dwWaitResult = WaitForSingleObject(hSyncThread, SYNC_TIMEOUT * 60 * 1000);      // 120 minutes timeout? useful?

    switch (dwWaitResult) {
        // Sync finished normally.
        case WAIT_ABANDONED:
        case WAIT_OBJECT_0: {
            break;
        }
        // Sync is running for SYNC_TIMEOUT minutes: terminate it.
        case WAIT_TIMEOUT: {
            // TODO: display a msgbox, terminate the sync, set exitcode, sendmsg, return.
            break;
        }
        // Some error occurred (case WAIT_FAILED)
        default: {
            // TODO: set exitcode, sendmsg, return.
            break;
        }
    }

    DWORD code = 0;
    GetExitCodeThread(hSyncThread, &code);
    SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_STARTSYNC_ENDED, NULL, (LPARAM)code);

    return 0;
}


/**
 * Thread used to kill the syncThread after a timeout.
 * @param lpParam : the syncThread HANDLE
 */
DWORD WINAPI syncThreadKiller(LPVOID lpParam) {

    // Wait on the sync thread (timeout = 5sec)
    int ret = 0;
    HANDLE hSyncThread = lpParam;
    DWORD dwWaitResult = WaitForSingleObject(hSyncThread, TIME_OUT_ABORT * 1000);

    switch (dwWaitResult) {
        // Thread exited -> no need to kill it (should be the usual way).
        case WAIT_ABANDONED:
        case WAIT_OBJECT_0: {
            ret = 0;
            break;
        }
        // Sync is still running after timeout -> kill thread.
        case WAIT_TIMEOUT: {
            hardTerminateSync(hSyncThread);
            ret = 0;
            break;
        }
        // Some error occurred (case WAIT_FAILED)
        default: {
            ret = 1;
            break;
        }
    }

    // To enable again the refresh of statusbar.
    cancelingSync = false;

    // Refresh status bar
    //CString s1;
    //s1.LoadString(IDS_TEXT_SYNC_CANCELED);
    //SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_REFRESH_STATUSBAR, (WPARAM)s1.GetString(), (LPARAM)s1.GetString());


    return ret;
}




void CMainSyncFrame::OnFileSynchronize() {
    if (getConfig()->getScheduledSync()) {
        // It's running a scheduled sync -> error msg.
        CString s1;
        s1.LoadString(IDS_TEXT_SYNC_ALREADY_RUNNING);
        MessageBox(s1);
    }
    else {
        this->StartSync();
    }
}


int CMainSyncFrame::OnCancelSync() {
    return 0;
}


void CMainSyncFrame::showSettingsWindow(){

    if(checkSyncInProgress()){
        CString s1;
        s1.LoadString(IDS_ERROR_CANNOT_CHANGE_SETTINGS);
        AfxMessageBox(s1);
        return;
    }

    CDocument* pDoc = NULL;
    pConfigFrame = NULL;

    CSingleDocTemplate* docSettings = ((COutlookPluginApp*)AfxGetApp())->docSettings;

    getConfig()->read();
    
    pDoc = docSettings->CreateNewDocument();
    if (pDoc != NULL)
    {
        // If creation worked, use create a new frame for
        // that document.
        pConfigFrame = (CConfigFrame*)docSettings->CreateNewFrame(pDoc, NULL);
        
        pConfigFrame->applyEnabled = false;

        if (pConfigFrame != NULL)
        {
            docSettings->SetDefaultTitle(pDoc);

            // If document initialization fails
            if (!pDoc->OnNewDocument())
            {
                pConfigFrame->DestroyWindow();
                pConfigFrame = NULL;
            }
            else
            {
                docSettings->InitialUpdateFrame(pConfigFrame, pDoc, TRUE);
            }
        }
    }

    // if it failed
    if (pConfigFrame == NULL || pDoc == NULL)
    {
        delete pDoc;
        AfxMessageBox(AFX_IDP_FAILED_TO_CREATE_DOC);
    } 

    pConfigFrame->wndSplitter.SetActivePane(0,0);
    pConfigFrame->wndSplitter.SetColumnInfo(0,65,40); // TODO: LeftView size
    pConfigFrame->wndSplitter.ReplaceView(0,1, RUNTIME_CLASS(CAccountSettings),CSize(100,100));
    pConfigFrame->wndSplitter.RecalcLayout();

    //select Account 
    ((CLeftView*)pConfigFrame->wndSplitter.GetPane(0,0))->lstConfig.SetSelectedColumn(0);
    
    pConfigFrame->wndSplitter.GetPane(0,1)->SendMessage(WM_PAINT);

    this->BeginModalState(); // TODO: this is required
    configOpened = true;
}

void CMainSyncFrame::OnToolsSetloglevel()
{
    // show the Log Level dialog
    CLogSettings wndLog;
    wndLog.DoModal();
}


LRESULT CMainSyncFrame::OnMsgSyncBegin( WPARAM , LPARAM lParam) {  

    CString s1;
    s1.LoadString(IDS_TEXT_STARTING_SYNC);
    wndStatusBar.SetPaneText(0,s1);

    if (!getConfig()->getScheduledSync()) {
        // hide the menu
        CMenu nullMenu;
        CString s1;
        nullMenu.LoadMenu(IDR_MAINFRAME_DISABLED);
        SetMenu(&nullMenu);
        nullMenu.Detach();

        ((CSyncForm*)wndSplitter.GetPane(0,1))->iconStatusSync.SetIcon(::LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_CANCEL)));
        s1.LoadString(IDS_MAIN_PRESS_TO_CANCEL); 
        ((CSyncForm*)wndSplitter.GetPane(0,1))->SetDlgItemText(IDC_MAIN_MSG_PRESS, s1);
    }
    else{
        // scheduled sync: keep black button
    }

    bSyncStarted = true;
    Invalidate();

    return 0;
}

// UI received a sync end message
LRESULT CMainSyncFrame::OnMsgSyncEnd( WPARAM , LPARAM ) {

    LOG.debug("msg syncEnd received by UI");
    CString s1;
    s1.LoadString(IDS_TEXT_SYNC_ENDED);
    wndStatusBar.SetPaneText(0,s1);

    bSyncStarted = false; 
    return 0;
}

// UI received sync source begin message
LRESULT CMainSyncFrame::OnMsgSyncSourceBegin( WPARAM wParam, LPARAM lParam)
{
    // TODO: blue highlight current sync pane
    CString s1;
    currentSource = lParam;
    currentItem = 0;
    
    // if it is scheduled, we change only status bar text
    bool isScheduled = getConfig()->getScheduledSync();

    if(!isScheduled){
        // change controls based on what source is currently syncing
        switch(currentSource){
            case SYNCSOURCE_CONTACTS:
                ((CSyncForm*)wndSplitter.GetPane(0,1))->GetDlgItem(IDC_MAIN_STATIC_CONTACTS)->ShowWindow(SW_SHOW);
                ((CSyncForm*)wndSplitter.GetPane(0,1))->GetDlgItem(IDC_MAIN_STATIC_STATUS_CONTACTS)->ShowWindow(SW_SHOW);
                ((CSyncForm*)wndSplitter.GetPane(0,1))->iconContacts.Animate();
                ((CSyncForm*)wndSplitter.GetPane(0,1))->iconStatusContacts.SetIcon(AfxGetApp()->LoadIcon(IDI_SYNC_ALL));
                ((CSyncForm*)wndSplitter.GetPane(0,1))->paneContacts.SetBitmap(LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BK_BLUE)));
                break;

            case SYNCSOURCE_CALENDAR:
                ((CSyncForm*)wndSplitter.GetPane(0,1))->GetDlgItem(IDC_MAIN_STATIC_CALENDAR)->ShowWindow(SW_SHOW);
                ((CSyncForm*)wndSplitter.GetPane(0,1))->GetDlgItem(IDC_MAIN_STATIC_STATUS_CALENDAR)->ShowWindow(SW_SHOW);
                ((CSyncForm*)wndSplitter.GetPane(0,1))->iconCalendar.Animate();
                ((CSyncForm*)wndSplitter.GetPane(0,1))->iconStatusCalendar.SetIcon(AfxGetApp()->LoadIcon(IDI_SYNC_ALL));
                ((CSyncForm*)wndSplitter.GetPane(0,1))->paneCalendar.SetBitmap(LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BK_BLUE)));
                break;

            case SYNCSOURCE_TASKS:
                ((CSyncForm*)wndSplitter.GetPane(0,1))->GetDlgItem(IDC_MAIN_STATIC_TASKS)->ShowWindow(SW_SHOW);
                ((CSyncForm*)wndSplitter.GetPane(0,1))->GetDlgItem(IDC_MAIN_STATIC_STATUS_TASKS)->ShowWindow(SW_SHOW);
                ((CSyncForm*)wndSplitter.GetPane(0,1))->iconTasks.Animate();
                ((CSyncForm*)wndSplitter.GetPane(0,1))->iconStatusTasks.SetIcon(AfxGetApp()->LoadIcon(IDI_SYNC_ALL));
                ((CSyncForm*)wndSplitter.GetPane(0,1))->paneTasks.SetBitmap(LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BK_BLUE)));
                break;

            case SYNCSOURCE_NOTES:
                ((CSyncForm*)wndSplitter.GetPane(0,1))->GetDlgItem(IDC_MAIN_STATIC_NOTES)->ShowWindow(SW_SHOW);
                ((CSyncForm*)wndSplitter.GetPane(0,1))->GetDlgItem(IDC_MAIN_STATIC_STATUS_NOTES)->ShowWindow(SW_SHOW);
                ((CSyncForm*)wndSplitter.GetPane(0,1))->iconNotes.Animate();
                ((CSyncForm*)wndSplitter.GetPane(0,1))->iconStatusNotes.SetIcon(AfxGetApp()->LoadIcon(IDI_SYNC_ALL));
                ((CSyncForm*)wndSplitter.GetPane(0,1))->paneNotes.SetBitmap(LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BK_BLUE)));
                break;
        };
    };

    syncSourceBegin ++;

    // change text on labels to reflect sync status
    CString msg; 
    if(wParam == 1){
        msg.LoadString(IDS_TEXT_RECEIVING);
    }
    else
        msg.LoadString(IDS_TEXT_SENDING);

    msg+=" ";

    
        
    CString source;
    switch(lParam){
        case SYNCSOURCE_CONTACTS:
            source.LoadString(IDS_TEXT_CONTACTS);
            msg+=source;
            if(!isScheduled)
                ((CSyncForm*)wndSplitter.GetPane(0,1))->changeContactsStatus(msg);
            ((CSyncForm*)wndSplitter.GetPane(0,1))->paneContacts.Invalidate();
            break;

        case SYNCSOURCE_CALENDAR:
            source.LoadString(IDS_TEXT_APPOINTMENTS);
            msg+=source;
            if(!isScheduled)
                ((CSyncForm*)wndSplitter.GetPane(0,1))->changeCalendarStatus(msg);
            ((CSyncForm*)wndSplitter.GetPane(0,1))->paneCalendar.Invalidate();
            break;

        case SYNCSOURCE_TASKS:
            source.LoadString(IDS_TEXT_TASKS);
            msg+=source;
            if(!isScheduled)
                ((CSyncForm*)wndSplitter.GetPane(0,1))->changeTasksStatus(msg);
            ((CSyncForm*)wndSplitter.GetPane(0,1))->paneTasks.Invalidate();
            break;

        case SYNCSOURCE_NOTES:
            source.LoadString(IDS_TEXT_NOTES);
            msg+=source;
            if(!isScheduled)
                ((CSyncForm*)wndSplitter.GetPane(0,1))->changeNotesStatus(msg);
            ((CSyncForm*)wndSplitter.GetPane(0,1))->paneNotes.Invalidate();
            break;
    };
    
    refreshStatusBar(msg); // set status bar text
    
    //if(!isScheduled)
      //  Invalidate(FALSE);
    return 0; 
}

// UI received a sync source ended message
LRESULT CMainSyncFrame::OnMsgSyncSourceEnd( WPARAM , LPARAM ) {

    CString s1;

    // if it is scheduled, we change only status bar text
    if(getConfig()->getScheduledSync())
        return 0;

    switch(currentSource){
     case SYNCSOURCE_CONTACTS:
         ((CSyncForm*)wndSplitter.GetPane(0,1))->iconContacts.StopAnim();
         ((CSyncForm*)wndSplitter.GetPane(0,1))->iconContacts.SetIcon(::LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_CONTACTS)));
         ((CSyncForm*)wndSplitter.GetPane(0,1))->paneContacts.SetBitmap(LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BK_LIGHT)));
         ((CSyncForm*)wndSplitter.GetPane(0,1))->iconStatusContacts.SetIcon(AfxGetApp()->LoadIcon(IDI_COMPLETE));
         s1.LoadString(IDS_DONE);
         ((CSyncForm*)wndSplitter.GetPane(0,1))->paneContacts.hPrevStatusIcon = AfxGetApp()->LoadIcon(IDI_COMPLETE);
         ((CSyncForm*)wndSplitter.GetPane(0,1))->changeContactsStatus(s1);
         ((CSyncForm*)wndSplitter.GetPane(0,1))->paneContacts.Invalidate();
         break;
     
     case SYNCSOURCE_CALENDAR:
         s1.LoadString(IDS_DONE);
         ((CSyncForm*)wndSplitter.GetPane(0,1))->iconCalendar.StopAnim();
         ((CSyncForm*)wndSplitter.GetPane(0,1))->iconCalendar.SetIcon(::LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_CALENDAR)));
         ((CSyncForm*)wndSplitter.GetPane(0,1))->paneCalendar.SetBitmap(LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BK_LIGHT)));
         ((CSyncForm*)wndSplitter.GetPane(0,1))->iconStatusCalendar.SetIcon(AfxGetApp()->LoadIcon(IDI_COMPLETE));
         ((CSyncForm*)wndSplitter.GetPane(0,1))->paneCalendar.hPrevStatusIcon = AfxGetApp()->LoadIcon(IDI_COMPLETE);
         ((CSyncForm*)wndSplitter.GetPane(0,1))->changeCalendarStatus(s1);
         ((CSyncForm*)wndSplitter.GetPane(0,1))->paneCalendar.Invalidate();
         break;
      
     case SYNCSOURCE_TASKS:
         s1.LoadString(IDS_DONE);
         ((CSyncForm*)wndSplitter.GetPane(0,1))->iconTasks.StopAnim();
         ((CSyncForm*)wndSplitter.GetPane(0,1))->iconTasks.SetIcon(::LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_TASKS)));
         ((CSyncForm*)wndSplitter.GetPane(0,1))->paneTasks.SetBitmap(LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BK_LIGHT)));
         ((CSyncForm*)wndSplitter.GetPane(0,1))->iconStatusTasks.SetIcon(AfxGetApp()->LoadIcon(IDI_COMPLETE));
         ((CSyncForm*)wndSplitter.GetPane(0,1))->paneTasks.hPrevStatusIcon = AfxGetApp()->LoadIcon(IDI_COMPLETE);
         ((CSyncForm*)wndSplitter.GetPane(0,1))->changeTasksStatus(s1);
         ((CSyncForm*)wndSplitter.GetPane(0,1))->paneTasks.Invalidate();
         break;

     case SYNCSOURCE_NOTES:
         s1.LoadString(IDS_DONE);
         ((CSyncForm*)wndSplitter.GetPane(0,1))->iconNotes.StopAnim();
         ((CSyncForm*)wndSplitter.GetPane(0,1))->iconNotes.SetIcon(::LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_NOTES)));
         ((CSyncForm*)wndSplitter.GetPane(0,1))->paneNotes.SetBitmap(LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BK_LIGHT)));
         ((CSyncForm*)wndSplitter.GetPane(0,1))->iconStatusNotes.SetIcon(AfxGetApp()->LoadIcon(IDI_COMPLETE));
         ((CSyncForm*)wndSplitter.GetPane(0,1))->paneNotes.hPrevStatusIcon = AfxGetApp()->LoadIcon(IDI_COMPLETE);
         ((CSyncForm*)wndSplitter.GetPane(0,1))->changeNotesStatus(s1);
         ((CSyncForm*)wndSplitter.GetPane(0,1))->paneNotes.Invalidate();
         break;

    };

    // TODO: check this
    //bSyncStarted = false;
    return 0;
}

// UI received a item synced message
LRESULT CMainSyncFrame::OnMsgItemSynced( WPARAM wParam, LPARAM )
{
    CString s1;

    currentItem++;
    CString statusBarText("");
    if(wParam == -1)
        statusBarText.LoadString(IDS_TEXT_SENDING);
    else
        statusBarText.LoadString(IDS_TEXT_RECEIVING);
    statusBarText += " ";


    // set status bar text
    statusBarText += " ";
    char* temp =  ltow(currentItem);
    statusBarText += temp;
    delete [] temp; temp = NULL;

    // '-1' received when #ofChanges is not supported.
    if(totalItems > 0){
        statusBarText += " of ";
        temp = ltow(totalItems);
        statusBarText += temp;
        delete [] temp; temp = NULL;
    }

    statusBarText += " ";

    // change status bar text
    switch(currentSource){
        case SYNCSOURCE_CONTACTS:
            s1.LoadString(IDS_TEXT_CONTACTS);
            statusBarText += s1;
            break;
        case SYNCSOURCE_CALENDAR:
            s1.LoadString(IDS_TEXT_APPOINTMENTS);
            statusBarText += s1;
            break;
        case SYNCSOURCE_TASKS:
            s1.LoadString(IDS_TEXT_TASKS);
            statusBarText += s1;
            break;
        case SYNCSOURCE_NOTES:
            s1.LoadString(IDS_TEXT_NOTES);
            statusBarText += s1;
            break;
    }
    
    refreshStatusBar(statusBarText);

    // if it is scheduled, we change only status bar text
    if(getConfig()->getScheduledSync()) {
        return 0;
    }

    // change source status
    switch(currentSource){
        case SYNCSOURCE_CONTACTS:
            ((CSyncForm*)wndSplitter.GetPane(0,1))->changeContactsStatus(statusBarText); 
            //((CSyncForm*)wndSplitter.GetPane(0,1))->repaintPaneControls(PANE_TYPE_CONTACTS);
            ((CSyncForm*)wndSplitter.GetPane(0,1))->paneContacts.Invalidate();
            break;
        case SYNCSOURCE_CALENDAR:
            ((CSyncForm*)wndSplitter.GetPane(0,1))->changeCalendarStatus(statusBarText); 
            ((CSyncForm*)wndSplitter.GetPane(0,1))->paneCalendar.Invalidate();
            break;
        case SYNCSOURCE_TASKS:
            ((CSyncForm*)wndSplitter.GetPane(0,1))->changeTasksStatus(statusBarText);    
            ((CSyncForm*)wndSplitter.GetPane(0,1))->paneTasks.Invalidate();
            break;
        case SYNCSOURCE_NOTES:
            ((CSyncForm*)wndSplitter.GetPane(0,1))->changeNotesStatus(statusBarText);
            ((CSyncForm*)wndSplitter.GetPane(0,1))->paneNotes.Invalidate();
            break;           
    }

    //Invalidate(FALSE);
    return 0;
}



afx_msg LRESULT CMainSyncFrame::OnMsgRefreshStatusBar( WPARAM wParam, LPARAM lParam) {

    char text[100];

    // TODO: move messages to UI resource!
    switch (lParam) {
        case SBAR_CHECK_ALL_ITEMS: {
            sprintf(text, SBAR_READING_ALLITEMS, (int)wParam);
            break;
        }
        case SBAR_CHECK_MOD_ITEMS: {
            sprintf(text, SBAR_CHECKING_MODITEMS);
            break;
        }
        case SBAR_CHECK_MOD_ITEMS2: {
            sprintf(text, SBAR_CHECKING_MODITEMS2, (int)wParam);
            break;
        }
        case SBAR_WRITE_OLD_ITEMS: {
            sprintf(text, SBAR_WRITING_OLDITEMS, (int)wParam);
            break;
        }
        case SBAR_SENDDATA_BEGIN:
        case SBAR_RECEIVE_DATA_BEGIN: {
            sprintf(text, SBAR_SYNCING_DATA, (int)wParam);
            break;
        }
        case SBAR_DELETE_CLIENT_ITEMS: {
            sprintf(text, SBAR_DELETING_ITEMS);
            break;
        }
    }

    CString s1 = text;
    refreshStatusBar(s1);

    return 0;
}



afx_msg LRESULT CMainSyncFrame::OnMsgTotalItems( WPARAM wParam, LPARAM lParam)
{
    totalItems = lParam;
   
    CString source;
    CString msg; 
    switch(currentSource){
        case SYNCSOURCE_CONTACTS:
            source.LoadString(IDS_TEXT_CONTACTS);
            break;
        case SYNCSOURCE_CALENDAR:
            source.LoadString(IDS_TEXT_APPOINTMENTS);
            break;
        case SYNCSOURCE_TASKS:
            source.LoadString(IDS_TEXT_TASKS);
            break;
        case SYNCSOURCE_NOTES:
            source.LoadString(IDS_TEXT_NOTES);
            break;
    };
    
    if(wParam == 1){
        msg.LoadString(IDS_TEXT_RECEIVING);
    }
    else
        msg.LoadString(IDS_TEXT_SENDING);
    msg+=" "; msg+=source;
    refreshStatusBar(msg);

    //Invalidate();
    return 0;
}

// the config window has closed, and the user is returned to the main window
void CMainSyncFrame::OnConfigClosed(){
    EndModalState();
    SetForegroundWindow();
    configOpened = false;
    ((CSyncForm*)wndSplitter.GetPane(0,1))->refreshSources();
};

LRESULT CMainSyncFrame::OnMsgTotalSources(WPARAM wParam, LPARAM lParam){
    return 0;
}


/**
 * Message received when sync thread has exited.
 * 'lParam' is the exitThread code (0 if no errors).
 * Here errors of sync process are managed, and then UI refreshed.
 */
LRESULT CMainSyncFrame::OnMsgStartsyncEnded(WPARAM wParam, LPARAM lParam){
    
    CString s1;
    _bstr_t bst;
    int exitCode = lParam;
    const bool isScheduled = getConfig()->getScheduledSync();
    
    //
    // Error occurred: display error message on a msgBox.
    //
    if (exitCode) {
        BeginModalState();

        manageSyncErrorMsg(exitCode);

        EndModalState();
        SetForegroundWindow();
    }
    else{
        // exitCode = 0 : sync finished ok
    }

    //
    // Open settings window if error on invalid credentials.
    //
    if ( (!isScheduled) &&
         (exitCode == 407  ||                   // 407  = Auth failed
          exitCode == 401  ||                   // 401  = Wrong credentials
          exitCode == 404  ||                   // 404  = not found
          exitCode == 2001 ||                   // 2001 = HTTP connection error
          exitCode == 2060) ) {                 // 2060 = HTTP resource not found (status 404)
        showSettingsWindow();
    }

    s1.LoadString(IDS_TEXT_SYNC_ENDED);
    refreshStatusBar(s1);

    ((CSyncForm*)wndSplitter.GetPane(0,1))->paneSync.SetBitmap(LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BK_DARK)));
    ((CSyncForm*)wndSplitter.GetPane(0,1))->iconStatusSync.SetIcon(::LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_SYNC_ALL_BLUE)));

    s1.LoadString(IDS_MAIN_PRESS_TO_SYNC); 
    ((CSyncForm*)wndSplitter.GetPane(0,1))->SetDlgItemText(IDC_MAIN_MSG_PRESS, s1);

    // show the menu
    CMenu appMenu;
    appMenu.LoadMenu(IDR_MAINFRAME);
    SetMenu(&appMenu); 
    UpdateWindow();
    appMenu.Detach();


    if (isScheduled) {
        // Scheduled sync: current config is out of date!
        getConfig()->read();
    }
    else {
        restoreSyncModeSettings();
    }

    // Refresh sources.
    ((CSyncForm*)wndSplitter.GetPane(0,1))->refreshSources();


    Invalidate(FALSE);
    bSyncStarted = false;
    return 0;
}

void CMainSyncFrame::StartSync(){

    CString s1;
    currentSource = 0;
    currentItem = 0;
    totalItems = 0;
    syncSourceBegin = 0;

    // Check on sync in progress.
    if (checkSyncInProgress()) {
        s1.LoadString(IDS_TEXT_SYNC_ALREADY_RUNNING);
        MessageBox(s1);
        return;
    }

    // Check if connection settings are valid.
    if(! checkConnectionSettings()) {
        s1.LoadString(IDS_ERROR_SET_CONNECTION);
        AfxMessageBox(s1);
        showSettingsWindow();
        return;
    }


    //
    // Start the sync thread.
    //
    bSyncStarted = true;
    cancelingSync = false;
    try {
        hSyncThread = CreateThread(NULL, 0, syncThread, 0, 0, &dwThreadId);
    }
    // *** To catch unexpected exceptions... ***
    catch(...){ 
        CString s1;
        s1.LoadString(IDS_ERROR_SYNC_NOT_COMPLETED);
        AfxMessageBox(s1);
        return;
    }

    //
    // Start the sync thread controller (manage sync exit).
    //
    DWORD ctrlThreadId;
    HANDLE h = CreateThread(NULL, 0, syncThreadController, hSyncThread, 0, &ctrlThreadId);


     // hide the menu
     CMenu nullMenu;
     nullMenu.LoadMenu(IDR_MAINFRAME_DISABLED);
     SetMenu(&nullMenu);
     nullMenu.Detach();

     // TODO: do not change main icon when sync has started
     ((CSyncForm*)wndSplitter.GetPane(0,1))->iconStatusSync.SetIcon(::LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_CANCEL)));

     s1.LoadString(IDS_MAIN_PRESS_TO_CANCEL); 
     //((CSyncForm*)wndSplitter.GetPane(0,1))->paneSync.SetBitmap(LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BK_DARK_BLUE)));
     ((CSyncForm*)wndSplitter.GetPane(0,1))->SetDlgItemText(IDC_MAIN_MSG_PRESS, s1);
     ((CSyncForm*)wndSplitter.GetPane(0,1))->refreshSources();
     ((CSyncForm*)wndSplitter.GetPane(0,1))->Invalidate();

     // do not show links during sync
     /*((CSyncForm*)wndSplitter.GetPane(0,1))->linkContacts.ShowWindow(SW_HIDE); 
     ((CSyncForm*)wndSplitter.GetPane(0,1))->linkCalendar.ShowWindow(SW_HIDE); 
     ((CSyncForm*)wndSplitter.GetPane(0,1))->linkTasks.ShowWindow(SW_HIDE); 
     ((CSyncForm*)wndSplitter.GetPane(0,1))->linkNotes.ShowWindow(SW_HIDE);*/

     // set state to panes
     ((CSyncForm*)wndSplitter.GetPane(0,1))->paneContacts.state = STATE_SYNC;
     ((CSyncForm*)wndSplitter.GetPane(0,1))->paneCalendar.state = STATE_SYNC;
     ((CSyncForm*)wndSplitter.GetPane(0,1))->paneTasks.state = STATE_SYNC;
     ((CSyncForm*)wndSplitter.GetPane(0,1))->paneNotes.state = STATE_SYNC;

}



int CMainSyncFrame::CancelSync(){
    int ret = 1;
    CString msg;
    CString s1;

    // This will avoid clicking 2 times on cancel sync.
    if (cancelingSync) {
        s1.LoadString(IDS_TEXT_SYNC_ALREADY_RUNNING);
        MessageBox(s1);
        return ret;
    }

    //
    // Display warning.
    //
    unsigned int flags = MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND | MB_APPLMODAL;
    int selected = MessageBox(WMSG_BOX_CANCEL_SYNC, WPROGRAM_NAME, flags);


    // First check again if sync is running (could be terminated in the meanwhile...)
    if (!checkSyncInProgress()) {
        return ret;
    }

    if (selected == IDYES) {
        ret = 0;

        // Refresh status bar
        CString s1;
        s1.LoadString(IDS_TEXT_CANCELING_SYNC);
        refreshStatusBar(s1);

        // LOCK the statusbar and other controls.
        cancelingSync = true;

        // First we try to terminate the sync in a soft way.
        softTerminateSync();
        
        // If it fails, this thread will kill the syncThread (after a timeout).
        DWORD killerThreadId;
        HANDLE h = CreateThread(NULL, 0, syncThreadKiller, hSyncThread, 0, &killerThreadId);

        bSyncStarted = false;

        //((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->detailsVisible = false;
        ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->OnNcPaint();

        // show the menu
        CMenu appMenu;
        appMenu.LoadMenu(IDR_MAINFRAME);
        SetMenu(&appMenu);
        appMenu.Detach();

        ((CSyncForm*)wndSplitter.GetPane(0,1))->refreshSources();

        ((CSyncForm*)wndSplitter.GetPane(0,1))->iconStatusSync.SetIcon(::LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_CANCEL)));
        s1.LoadString(IDS_MAIN_PRESS_TO_SYNC); 
        ((CSyncForm*)wndSplitter.GetPane(0,1))->SetDlgItemText(IDC_MAIN_MSG_PRESS, s1);

        // set sync source status
        if(strcmp(getConfig()->getSyncSourceConfig(CONTACT_)->getSync(),"none") != 0)
            ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->syncSourceContactState = SYNCSOURCE_STATE_CANCELED;
        if(strcmp(getConfig()->getSyncSourceConfig(APPOINTMENT_)->getSync(),"none") != 0)
            ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->syncSourceCalendarState = SYNCSOURCE_STATE_CANCELED;
        if(strcmp(getConfig()->getSyncSourceConfig(TASK_)->getSync(),"none") != 0)
            ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->syncSourceTaskState = SYNCSOURCE_STATE_CANCELED;
        if(strcmp(getConfig()->getSyncSourceConfig(NOTE_)->getSync(),"none") != 0)
            ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->syncSourceNoteState = SYNCSOURCE_STATE_CANCELED;

        //
        // ***TODO*** is this call necessary?
        // Restore of syncModes is (always) called on "OnMsgStartsyncEnded()"
        //
        restoreSyncModeSettings(); // restore changes made by clicking 'sync' link

        Invalidate();
        currentSource = 0;
        currentItem = 0;
        totalItems = 0;
        syncSourceBegin = 0;
    }    
    
    return ret;    
}

BOOL CMainSyncFrame::OnNcActivate(BOOL bActive){
    // needs special handling only when the config window is opened
    if(configOpened){
        if( (bActive) && (pConfigFrame != NULL))
            if( (! this->IsWindowEnabled()) && 
                (pConfigFrame->IsWindowVisible() ) 
                //(pConfigFrame->IsWindowVisible()) //&&
                //(pConfigFrame->IsWindowEnabled())
                )
            {
                pConfigFrame->SetForegroundWindow();
                pConfigFrame->SetFocus();
            };
    };

    CFrameWnd::OnNcActivate(bActive);
    return TRUE;
} 

void CMainSyncFrame::OnClose(){

    // CancelSync only if sync in progress AND not a scheduled one!
    // (if scheduled, the sync will continue in bkground)
    if( (checkSyncInProgress()) && (!getConfig()->getScheduledSync()) ) {

        if (CancelSync()) {
            // cancelled
            return;
        }
    }

    closeClient();
    CFrameWnd::OnClose();
}


void CMainSyncFrame::backupSyncModeSettings(){
    syncModeContacts = getSyncModeCode(getConfig()->getSyncSourceConfig(CONTACT_)->getSync());
    syncModeCalendar = getSyncModeCode(getConfig()->getSyncSourceConfig(APPOINTMENT_)->getSync());
    syncModeTasks = getSyncModeCode(getConfig()->getSyncSourceConfig(TASK_)->getSync());
    syncModeNotes = getSyncModeCode(getConfig()->getSyncSourceConfig(NOTE_)->getSync());
}

void CMainSyncFrame::restoreSyncModeSettings(){

    if(syncModeContacts != -1)
        getConfig()->getSyncSourceConfig(CONTACT_)->setSync(syncModeName((SyncMode)syncModeContacts));
    if(syncModeCalendar != -1)
        getConfig()->getSyncSourceConfig(APPOINTMENT_)->setSync(syncModeName((SyncMode)syncModeCalendar));
    if(syncModeTasks    != -1)
        getConfig()->getSyncSourceConfig(TASK_)->setSync(syncModeName((SyncMode)syncModeTasks));
    if(syncModeNotes    != -1)
        getConfig()->getSyncSourceConfig(NOTE_)->setSync(syncModeName((SyncMode)syncModeNotes));

    // Save ONLY sync-modes of each source, if necessary.
    if ( syncModeContacts != -1 || 
         syncModeCalendar != -1 ||
         syncModeTasks    != -1 ||
         syncModeNotes    != -1 ) {
        getConfig()->saveSyncModes();
    }
    
    syncModeContacts = -1; syncModeCalendar = -1; syncModeTasks = -1; syncModeNotes = -1;
}

LRESULT CMainSyncFrame::OnMsgAppModal(WPARAM, LPARAM){
    if(InModalState()){
        EndModalState();
        SetForegroundWindow();
    }
    else
        BeginModalState();
    
    return 0;
}

bool CMainSyncFrame::checkConnectionSettings()
{
    bool isOk = true;
    CString s1;

    // first check if the server URL is configured
    s1 = getConfig()->getAccessConfig().getSyncURL();
    if (s1.Find(_T("<")) > 0)
        isOk = false;

    if( (strcmp(getConfig()->getAccessConfig().getUsername(),"") == 0) ||
        (strcmp(getConfig()->getAccessConfig().getPassword(),"") == 0) )
        isOk = false;

    return isOk;
}


LRESULT CMainSyncFrame::OnMsgSyncSourceState(WPARAM wParam, LPARAM lParam) {

    if(wParam == ID_CONTACT) {
        ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->syncSourceContactState = lParam;
        
        // Update the status icon (funzilla #2110)
        if (lParam == SYNCSOURCE_STATE_OK)
            ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->iconStatusContacts.SetIcon(AfxGetApp()->LoadIcon(IDI_OK));
        else 
            ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->iconStatusContacts.SetIcon(AfxGetApp()->LoadIcon(IDI_ALERT));
    }

    if(wParam == ID_APPOINTMENT) {
        ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->syncSourceCalendarState = lParam;
        // Update the status icon (funzilla #2110)
        if (lParam == SYNCSOURCE_STATE_OK)
            ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->iconStatusCalendar.SetIcon(AfxGetApp()->LoadIcon(IDI_OK));
        else 
            ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->iconStatusCalendar.SetIcon(AfxGetApp()->LoadIcon(IDI_ALERT));
    }

    if(wParam == ID_TASK) {
        ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->syncSourceTaskState = lParam;
        // Update the status icon (funzilla #2110)
        if (lParam == SYNCSOURCE_STATE_OK)
            ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->iconStatusTasks.SetIcon(AfxGetApp()->LoadIcon(IDI_OK));
        else 
            ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->iconStatusTasks.SetIcon(AfxGetApp()->LoadIcon(IDI_ALERT));
    }

    if(wParam == ID_NOTE) {
        ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->syncSourceNoteState = lParam;
        // Update the status icon (funzilla #2110)
        if (lParam == SYNCSOURCE_STATE_OK)
            ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->iconStatusNotes.SetIcon(AfxGetApp()->LoadIcon(IDI_OK));
        else 
            ((CSyncForm*)((CMainSyncFrame*)AfxGetMainWnd())->wndSplitter.GetPane(0,1))->iconStatusNotes.SetIcon(AfxGetApp()->LoadIcon(IDI_ALERT));

    }

    return 0;
}