/*
* Copyright (C) 2003-2007 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "base/fscapi.h"
#include "base/Log.h"
#include "winmaincpp.h"
#include "outlook/defs.h"

#include "outlook/ClientMail.h"
#include "outlook/ClientException.h"
#include "outlook/utils.h"


using namespace std;




// Constructor
ClientMail::ClientMail() : ClientItem() {

    pMail           = NULL;
    pSafeMail       = NULL;

    // Redemption objects are initialized only 
    // when required: speed up...
    //createSafeMailInstance();
}

// Destructor
ClientMail::~ClientMail() {
}




/*
 * Set a COM pointer to this object.
 * ------------------------------------
 * This method is used to link the object to the correspondent
 * outlook COM pointer. All class members are overwrited by this call.
 * The method MUST be called before using this object, as the constructor
 * doesn't link the class COM pointer.
 * This method also links Redemption COM pointer to the item.
 * If 'itemID' parameter is an empty string, then ID is derived
 * from the entryID of the outlook item.
 */
void ClientMail::setCOMPtr(_MailItemPtr& ptr, const wstring& itemID) {

    pMail = ptr;

    try {
        pItemProperties = pMail->ItemProperties;
        propertiesCount = pItemProperties->Count;
        if (propertiesCount) {
            pItemProperty = pItemProperties->Item(0);
        }
        // ID passed as parameter
        if (itemID != EMPTY_WSTRING) {
            ID = itemID;
        }
        // ID retrieved from Outlook
        else {
            _bstr_t bstrID = pMail->GetEntryID();
            // ID could not exist (if item not saved)
            if (bstrID.GetBSTR()) {
                ID = (WCHAR*)bstrID;
            }
            else {
                ID = EMPTY_WSTRING;
            }
        }
        // full path of parent folder
        MAPIFolderPtr parentFolder = (MAPIFolderPtr)pMail->GetParent();
        parentPath = (WCHAR*)parentFolder->GetFullFolderPath();
    }
    catch(_com_error &e) {
        manageComErrors(e);
        throwClientFatalException(ERR_OUTLOOK_ITEM_ASSIGN);
    }

    hr              = S_OK;
    itemType        = MAIL;
    propertiesIndex = 0;

    // Assume all items of this type have same props!
    //propertyMap.clear();
}


// Here ID is derived from the entryID of the item.
void ClientMail::setCOMPtr(_MailItemPtr& ptr) {
    return setCOMPtr(ptr, EMPTY_WSTRING);
}



/*
 * Creates an instance for the Redemption COM pointer.
 */
void ClientMail::createSafeMailInstance() {

    try {
        hr = pSafeMail.CreateInstance(RED_SAFEMAIL);
        if (FAILED(hr)) {
            throwClientFatalException(ERR_OUTLOOK_SAFEITEM);
        }
    }
    catch(_com_error &e) {
        manageComErrors(e);
        throwClientFatalException(ERR_OUTLOOK_SAFEITEM);
    }
}





/*
 * Save the current Item.
 * @return: 0 if no errors.
 */
int ClientMail::saveItem() {
    
    try {
        hr = pMail->Save();
        if (FAILED(hr))
            goto error;

        // Get the new entry-ID.
        ID = (WCHAR*)pMail->GetEntryID();
    }
    catch(_com_error &e) {
        manageComErrors(e);
        goto error;
    }

    return 0;

error:
    sprintf(lastErrorMsg, ERR_OUTLOOK_ITEM_SAVE, itemType.c_str());
    throwClientException(lastErrorMsg);
    return 1;
}


/*
 * Delete the current Item.
 * @return: 0 if no errors.
 */
int ClientMail::deleteItem() {
    
    try {
        hr = pMail->Delete();
        if (FAILED(hr)) {
            goto error;
        }
    }
    catch(_com_error &e) {
        manageComErrors(e);
        goto error;
    }

    return 0;

error:
    sprintf(lastErrorMsg, ERR_OUTLOOK_ITEM_DELETE, itemType.c_str(), ID.c_str());
    throwClientException(lastErrorMsg);
    return 1;
}



/**
 * Returns a (new allocated) copy of this item.
 * @return : a pointer to the new ClientItem object (must bee deleted by the caller).
 *           NULL in case of errors.
 */
ClientItem* ClientMail::copyItem() {

    IDispatchPtr  pNew    = NULL;
    _MailItemPtr  pMaiNew = NULL;
    ClientMail*   cNew    = NULL;

    if (!pMail) {
        goto error;
    }

    try {
        // Copy the COM pointer
        pNew = pMail->Copy();
        if (!pNew) {
            goto error;
        }
        pMaiNew = (_MailItemPtr)pNew;
        if (!pMaiNew) {
            goto error;
        }
    }
    catch(_com_error &e) {
        manageComErrors(e);
        goto error;
    }

    // Set the new COM pointer to a new allocated ClientMail.
    cNew = new ClientMail();
    cNew->setCOMPtr(pMaiNew);

    return (ClientItem*)cNew;

error:
    return NULL;
}




//
// ------------------------------ Methods to manage item properties -----------------------------
//


bool ClientMail::isSecureProperty(const wstring& propertyName) {
    return false;
}

bool ClientMail::isComplexProperty(const wstring& propertyName) {
    return false;
}

const wstring ClientMail::getSafeProperty(const wstring& propertyName) {
    return NULL;
}

const wstring ClientMail::getComplexProperty(const wstring& propertyName) {
    return NULL;
}




//const wstring ClientMail::getFirstProperty() {
//    return NULL;
//}
//const wstring ClientMail::getNextProperty() {
//    return NULL;
//}
//const wstring ClientMail::getPreviousProperty() {
//    return NULL;
//}
//const wstring ClientMail::getLastProperty() {
//    return NULL;
//}


//int ClientMail::setProperty(const wstring& propertyName, const wstring& propertyValue) {
//    return NULL;
//}

int ClientMail::setComplexProperty(const wstring& propertyName, const wstring& propertyValue) {
    return 0;
}


/* 
 * Copy Constructor
*/
ClientMail::ClientMail(const ClientMail& c) {
    hr              = c.hr;
    ID              = c.ID;
    itemType        = c.itemType;
    parentPath      = c.parentPath;
    pMail           = c.pMail;
    pItemProperties = c.pItemProperties;
    pItemProperty   = c.pItemProperty;
    propertiesIndex = c.propertiesIndex;
    propertiesCount = c.propertiesCount;
    propertyMap     = c.propertyMap;

    pSafeMail       = NULL;
}


/* 
 * Operator =
*/
ClientMail ClientMail::operator=(const ClientMail& c) {

    ClientMail cnew;

    cnew.hr              = c.hr;
    cnew.ID              = c.ID;
    cnew.itemType        = c.itemType;
    cnew.parentPath      = c.parentPath;
    cnew.pMail           = c.pMail;
    cnew.pItemProperties = c.pItemProperties;
    cnew.pItemProperty   = c.pItemProperty;
    cnew.propertiesIndex = c.propertiesIndex;
    cnew.propertiesCount = c.propertiesCount;
    cnew.propertyMap     = c.propertyMap;

    cnew.pSafeMail       = NULL;

    return cnew;
}

