/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef INCL_UTILS_DLL
#define INCL_UTILS_DLL


#include "base/fscapi.h"
#include "spds/SyncReport.h"
#include "WindowsSyncSource.h"
#include "outlook/ClientRecurrence.h"

#include <string>

#define SYNCSOURCE_CALENDAR     0
#define SYNCSOURCE_CONTACTS     1
#define SYNCSOURCE_NOTES        2
#define SYNCSOURCE_TASKS        3


/*-----------utils functions -----------------*/

// Open a message box with a timeout
int TimedMessageBox(HWND hwndOwner, LPCTSTR pszMessage, LPCTSTR pszTitle, UINT flags, DWORD dwTimeout);
// Display a message box only if not scheduled sync.
int safeMessageBox (const char*   message, const char*   title = NULL, unsigned int flags = 0);
int wsafeMessageBox(const WCHAR* wmessage, const WCHAR* wtitle = NULL, unsigned int flags = 0);

bool  isSIF            (const std::wstring& dataType);
bool  isSIF            (const std::string&  dataType);
bool isAcceptedDataType(const std::wstring& dataType);
bool isAcceptedDataType(const std::string&  dataType);
char* syncModeName     (SyncMode code);
bool isFullSyncMode    (SyncMode mode);

void  toWindows        (char* str);
void  replaceAll       (const std::wstring& source, const std::wstring& dest, std::wstring& dataString);
int   getElementContent(const std::wstring& xml, const std::wstring& tag, std::wstring& content, unsigned int pos = 0);
int   getElementContent(const std::wstring& xml, const std::wstring& tag, std::wstring& content, const std::wstring::size_type pos, std::wstring::size_type& start, std::wstring::size_type& end);

WCHAR* readAppDataPath();
WCHAR* readDataPath   (const WCHAR* itemType);
int    makeDataDirs   ();

std::wstring readFromFile          (const std::wstring& filePath);
int          writeToFile           (const std::wstring& content, const std::wstring& filePath, const WCHAR* mode = L"w");
int          writeToFile           (const std::string&  content, const std::string&  filePath, const char*  mode = "w");
std::string  getSyncMutexName      ();
char*        readSystemErrorMsg    (DWORD errorCode = 0);
std::wstring getSafeItemName       (ClientItem* cItem);
void         printReport           (SyncReport* sr, WindowsSyncSource** sources);
char*        friendlyName          (const char* sourceName);

long         variantTimeToTimeStamp       (const double vTime);
BOOL         isAllDayEventInterval        (const DATE startdate, const DATE enddate);
void         doubleToSystemTimeDateForVCal(std::wstring& systemDate, const DATE doubleDate);
std::wstring clientRecurrenceToRRULE      (ClientRecurrence* cRec);
bool         rRULEToClientRecurrence      (const std::wstring& rrule, ClientRecurrence* cRec, const DATE startDate);
WCHAR*       extractDayOfWeek             (long l);
int          calculateDayOfWeek           (WCHAR* in);
bool         isDayOfWeek                  (WCHAR* data);
long         getWeekDay                   (DATE dt);

int syncSourceNameToIndex(const char*);



/**
 * Is the given status code an error status code? Error codes are the ones
 * outside the range 200-299.
 *
 * @param status the status code to check
 */
inline static bool isErrorStatus(int status) {
    return (status) && ((status < 200) || (status > 299));
}


#endif